package Catalyst::Runtime;

use strict;
use warnings;

BEGIN { require 5.008004; }

# Remember to update this in Catalyst as well!

our $VERSION='5.80014_01';

$VERSION = eval $VERSION;

=head1 NAME

Catalyst::Runtime - The Catalyst Framework Runtime

=head1 SYNOPSIS

See L<Catalyst>.

=head1 DESCRIPTION

This is the primary class for the Catalyst-Runtime distribution, version 5.80.

=head1 AUTHORS

Catalyst Contributors, see Catalyst.pm

=head1 COPYRIGHT

This library is free software. You can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

1;
